var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import { enrichEventWithDetails, ThemingParameters, useIsRTL, useSyncRef } from '@ui5/webcomponents-react-base';
import React, { forwardRef, useCallback } from 'react';
import { Bar, BarChart as BarChartLib, Brush, CartesianGrid, Cell, Label, LabelList, Legend, ReferenceLine, Tooltip, XAxis, YAxis } from 'recharts';
import { getValueByDataKey } from 'recharts/lib/util/ChartUtils';
import { useCancelAnimationFallback } from '../../hooks/useCancelAnimationFallback';
import { useChartMargin } from '../../hooks/useChartMargin';
import { useLabelFormatter } from '../../hooks/useLabelFormatter';
import { useLegendItemClick } from '../../hooks/useLegendItemClick';
import { useLongestYAxisLabelBar } from '../../hooks/useLongestYAxisLabelBar';
import { useObserveXAxisHeights } from '../../hooks/useObserveXAxisHeights';
import { useOnClickInternal } from '../../hooks/useOnClickInternal';
import { usePrepareDimensionsAndMeasures } from '../../hooks/usePrepareDimensionsAndMeasures';
import { useTooltipFormatter } from '../../hooks/useTooltipFormatter';
import { ChartContainer } from '../../internal/ChartContainer';
import { ChartDataLabel } from '../../internal/ChartDataLabel';
import { defaultFormatter } from '../../internal/defaults';
import { tickLineConfig, tooltipContentStyle, tooltipFillOpacity } from '../../internal/staticProps';
import { getCellColors, resolvePrimaryAndSecondaryMeasures } from '../../internal/Utils';
import { XAxisTicks } from '../../internal/XAxisTicks';
import { YAxisTicks } from '../../internal/YAxisTicks';
import { BarChartPlaceholder } from './Placeholder';
const dimensionDefaults = {
    formatter: defaultFormatter
};
const measureDefaults = {
    formatter: defaultFormatter,
    opacity: 1
};
const valueAccessor = (attribute) => ({ payload }) => {
    return getValueByDataKey(payload, attribute);
};
/**
 * A `BarChart` is a data visualization where each category is represented by a rectangle, with the width of the rectangle being proportional to the values being plotted.
 */
const BarChart = forwardRef((props, ref) => {
    var _a, _b, _c, _d, _e;
    const { loading, dataset, noLegend, noAnimation, tooltipConfig, onDataPointClick, onLegendClick, onClick, style, className, slot, syncId, ChartPlaceholder, children } = props, rest = __rest(props, ["loading", "dataset", "noLegend", "noAnimation", "tooltipConfig", "onDataPointClick", "onLegendClick", "onClick", "style", "className", "slot", "syncId", "ChartPlaceholder", "children"]);
    const chartConfig = Object.assign({ margin: {}, yAxisVisible: true, xAxisVisible: true, gridStroke: ThemingParameters.sapList_BorderColor, gridHorizontal: true, gridVertical: false, legendPosition: 'bottom', legendHorizontalAlign: 'left', barGap: 3, zoomingTool: false, resizeDebounce: 250, yAxisConfig: {}, xAxisConfig: {}, secondXAxisConfig: {} }, props.chartConfig);
    const { dimensions, measures } = usePrepareDimensionsAndMeasures(props.dimensions, props.measures, dimensionDefaults, measureDefaults);
    const tooltipValueFormatter = useTooltipFormatter(measures);
    const primaryDimension = dimensions[0];
    const { primaryMeasure, secondaryMeasure } = resolvePrimaryAndSecondaryMeasures(measures, (_a = chartConfig === null || chartConfig === void 0 ? void 0 : chartConfig.secondYAxis) === null || _a === void 0 ? void 0 : _a.dataKey);
    const dataKeys = measures.map(({ accessor }) => accessor);
    const colorSecondY = chartConfig.secondYAxis
        ? dataKeys.findIndex((key) => { var _a; return key === ((_a = chartConfig.secondYAxis) === null || _a === void 0 ? void 0 : _a.dataKey); })
        : 0;
    const [componentRef, chartRef] = useSyncRef(ref);
    const onItemLegendClick = useLegendItemClick(onLegendClick);
    const labelFormatter = useLabelFormatter(primaryDimension);
    const onDataPointClickInternal = useCallback((payload, i, event) => {
        if (payload && onDataPointClick) {
            const value = payload.value.length ? payload.value[1] - payload.value[0] : payload.value;
            onDataPointClick(enrichEventWithDetails(event, {
                dataKey: Object.keys(payload)
                    .filter((key) => key !== 'value')
                    .find((key) => payload[key] === value),
                value,
                payload: payload.payload,
                dataIndex: i
            }));
        }
    }, [onDataPointClick]);
    const onClickInternal = useOnClickInternal(onClick);
    const isBigDataSet = (dataset === null || dataset === void 0 ? void 0 : dataset.length) > 30;
    const primaryDimensionAccessor = primaryDimension === null || primaryDimension === void 0 ? void 0 : primaryDimension.accessor;
    const [width, legendPosition] = useLongestYAxisLabelBar(dataset, dimensions);
    const marginChart = useChartMargin(chartConfig.margin, chartConfig.zoomingTool);
    const [xAxisHeight] = useObserveXAxisHeights(chartRef, 1);
    const isRTL = useIsRTL(chartRef);
    const { isMounted, handleBarAnimationStart, handleBarAnimationEnd } = useCancelAnimationFallback(noAnimation);
    const { chartConfig: _0, dimensions: _1, measures: _2 } = rest, propsWithoutOmitted = __rest(rest, ["chartConfig", "dimensions", "measures"]);
    return (React.createElement(ChartContainer, Object.assign({ dataset: dataset, loading: loading, Placeholder: ChartPlaceholder !== null && ChartPlaceholder !== void 0 ? ChartPlaceholder : BarChartPlaceholder, ref: componentRef, style: style, className: className, slot: slot, resizeDebounce: chartConfig.resizeDebounce }, propsWithoutOmitted),
        React.createElement(BarChartLib, { syncId: syncId, onClick: onClickInternal, stackOffset: "sign", margin: marginChart, layout: "vertical", data: dataset, barGap: chartConfig.barGap, className: typeof onDataPointClick === 'function' || typeof onClick === 'function' ? 'has-click-handler' : undefined },
            React.createElement(CartesianGrid, { vertical: chartConfig.gridVertical, horizontal: chartConfig.gridHorizontal, stroke: chartConfig.gridStroke }),
            chartConfig.xAxisVisible && (React.createElement(XAxis, Object.assign({ interval: 0, type: "number", tick: React.createElement(XAxisTicks, { config: primaryMeasure }), axisLine: chartConfig.xAxisVisible, tickLine: tickLineConfig, tickFormatter: primaryMeasure === null || primaryMeasure === void 0 ? void 0 : primaryMeasure.formatter, height: xAxisHeight, reversed: isRTL }, chartConfig.xAxisConfig))),
            ((_b = chartConfig.secondYAxis) === null || _b === void 0 ? void 0 : _b.dataKey) && (React.createElement(XAxis, Object.assign({ dataKey: chartConfig.secondYAxis.dataKey, axisLine: {
                    stroke: (_c = chartConfig.secondYAxis.color) !== null && _c !== void 0 ? _c : `var(--sapChart_OrderedColor_${(colorSecondY % 11) + 1})`
                }, tick: React.createElement(XAxisTicks, { config: secondaryMeasure, secondYAxisConfig: {
                        color: (_d = chartConfig.secondYAxis.color) !== null && _d !== void 0 ? _d : `var(--sapChart_OrderedColor_${(colorSecondY % 11) + 1})`
                    } }), tickLine: {
                    stroke: (_e = chartConfig.secondYAxis.color) !== null && _e !== void 0 ? _e : `var(--sapChart_OrderedColor_${(colorSecondY % 11) + 1})`
                }, 
                // eslint-disable-next-line @typescript-eslint/ban-ts-comment
                // @ts-ignore
                label: { value: chartConfig.secondYAxis.name, offset: 2, angle: +90, position: 'center' }, orientation: "top", interval: 0, xAxisId: "secondary", type: "number" }, chartConfig.secondXAxisConfig))),
            chartConfig.yAxisVisible &&
                dimensions.map((dimension, index) => {
                    var _a;
                    return (React.createElement(YAxis, Object.assign({ interval: (_a = dimension === null || dimension === void 0 ? void 0 : dimension.interval) !== null && _a !== void 0 ? _a : (isBigDataSet ? 'preserveStart' : 0), type: "category", key: dimension.accessor, dataKey: dimension.accessor, tick: React.createElement(YAxisTicks, { config: dimension }), tickLine: index < 1, axisLine: index < 1, yAxisId: index, width: width[index], allowDuplicatedCategory: index === 0, orientation: isRTL ? 'right' : 'left' }, chartConfig.yAxisConfig)));
                }),
            isMounted &&
                measures.map((element, index) => {
                    var _a, _b, _c;
                    return (React.createElement(Bar, { stackId: element.stackId, fillOpacity: element.opacity, key: element.accessor, name: (_a = element.label) !== null && _a !== void 0 ? _a : element.accessor, strokeOpacity: element.opacity, type: "monotone", dataKey: element.accessor, fill: (_b = element.color) !== null && _b !== void 0 ? _b : `var(--sapChart_OrderedColor_${(index % 11) + 1})`, stroke: (_c = element.color) !== null && _c !== void 0 ? _c : `var(--sapChart_OrderedColor_${(index % 11) + 1})`, barSize: element.width, 
                        // eslint-disable-next-line @typescript-eslint/ban-ts-comment
                        // @ts-ignore
                        onClick: onDataPointClickInternal, isAnimationActive: noAnimation === false, onAnimationStart: handleBarAnimationStart, onAnimationEnd: handleBarAnimationEnd },
                        React.createElement(LabelList, { data: dataset, valueAccessor: valueAccessor(element.accessor), content: React.createElement(ChartDataLabel, { config: element, chartType: "bar", position: 'insideRight' }) }),
                        dataset.map((data, i) => {
                            return (React.createElement(Cell, { key: i, fill: getCellColors(element, data, index), stroke: getCellColors(element, data, index) }));
                        })));
                }),
            !noLegend && (
            // eslint-disable-next-line @typescript-eslint/ban-ts-comment
            // @ts-ignore
            React.createElement(Legend, { verticalAlign: chartConfig.legendPosition, align: chartConfig.legendHorizontalAlign, onClick: onItemLegendClick, wrapperStyle: legendPosition })),
            chartConfig.referenceLine && (React.createElement(ReferenceLine, { stroke: chartConfig.referenceLine.color, x: chartConfig.referenceLine.value },
                React.createElement(Label, null, chartConfig.referenceLine.label))),
            (tooltipConfig === null || tooltipConfig === void 0 ? void 0 : tooltipConfig.active) !== false && (React.createElement(Tooltip, Object.assign({ cursor: tooltipFillOpacity, formatter: tooltipValueFormatter, contentStyle: tooltipContentStyle, labelFormatter: labelFormatter }, tooltipConfig))),
            chartConfig.zoomingTool && (React.createElement(Brush, { y: 10, dataKey: primaryDimensionAccessor, tickFormatter: primaryDimension.formatter, stroke: ThemingParameters.sapObjectHeader_BorderColor, travellerWidth: 10, height: 20 })),
            children)));
});
BarChart.defaultProps = {
    noLegend: false,
    noAnimation: false
};
BarChart.displayName = 'BarChart';
export { BarChart };
