var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import { enrichEventWithDetails, ThemingParameters, useIsRTL, useSyncRef } from '@ui5/webcomponents-react-base';
import React, { forwardRef, useCallback, useMemo } from 'react';
import { Bar, Brush, CartesianGrid, Cell, ComposedChart as ComposedChartLib, Label, Legend, ReferenceLine, Tooltip, XAxis, YAxis } from 'recharts';
import { useChartMargin } from '../../hooks/useChartMargin';
import { useLabelFormatter } from '../../hooks/useLabelFormatter';
import { useLegendItemClick } from '../../hooks/useLegendItemClick';
import { useLongestYAxisLabel } from '../../hooks/useLongestYAxisLabel';
import { useObserveXAxisHeights } from '../../hooks/useObserveXAxisHeights';
import { useOnClickInternal } from '../../hooks/useOnClickInternal';
import { usePrepareDimensionsAndMeasures } from '../../hooks/usePrepareDimensionsAndMeasures';
import { useTooltipFormatter } from '../../hooks/useTooltipFormatter';
import { ChartContainer } from '../../internal/ChartContainer';
import { ChartDataLabel } from '../../internal/ChartDataLabel';
import { defaultFormatter } from '../../internal/defaults';
import { tickLineConfig, tooltipContentStyle, tooltipFillOpacity } from '../../internal/staticProps';
import { getCellColors, resolvePrimaryAndSecondaryMeasures } from '../../internal/Utils';
import { XAxisTicks } from '../../internal/XAxisTicks';
import { YAxisTicks } from '../../internal/YAxisTicks';
import { ComparisonLine } from './ComparisonLine';
import { BulletChartPlaceholder } from './Placeholder';
const dimensionDefaults = {
    formatter: defaultFormatter
};
const measureDefaults = {
    formatter: defaultFormatter,
    opacity: 1
};
/**
 * The `BulletChart` is used to compare primary and secondary (comparison) values. The primary and additional values
 * are rendered as a stacked Bar Chart while the comparison value is displayed as a line above.
 */
const BulletChart = forwardRef((props, ref) => {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k;
    const { loading, dataset, onDataPointClick, noLegend, noAnimation, tooltipConfig, onLegendClick, onClick, layout, style, className, slot, syncId, ChartPlaceholder, children } = props, rest = __rest(props, ["loading", "dataset", "onDataPointClick", "noLegend", "noAnimation", "tooltipConfig", "onLegendClick", "onClick", "layout", "style", "className", "slot", "syncId", "ChartPlaceholder", "children"]);
    const [componentRef, chartRef] = useSyncRef(ref);
    const chartConfig = Object.assign({ yAxisVisible: false, xAxisVisible: true, gridStroke: ThemingParameters.sapList_BorderColor, gridHorizontal: true, gridVertical: false, legendPosition: 'bottom', legendHorizontalAlign: 'left', zoomingTool: false, resizeDebounce: 250, yAxisConfig: {}, xAxisConfig: {}, secondYAxisConfig: {}, secondXAxisConfig: {} }, props.chartConfig);
    const { dimensions, measures } = usePrepareDimensionsAndMeasures(props.dimensions, props.measures, dimensionDefaults, measureDefaults);
    const sortedMeasures = useMemo(() => {
        return measures.sort((measure) => {
            if (measure.type === 'comparison') {
                return 1;
            }
            if (measure.type === 'primary') {
                return -1;
            }
            return 0;
        });
    }, [measures]);
    const tooltipValueFormatter = useTooltipFormatter(sortedMeasures);
    const primaryDimension = dimensions[0];
    const { primaryMeasure, secondaryMeasure } = resolvePrimaryAndSecondaryMeasures(sortedMeasures, (_a = chartConfig === null || chartConfig === void 0 ? void 0 : chartConfig.secondYAxis) === null || _a === void 0 ? void 0 : _a.dataKey);
    const labelFormatter = useLabelFormatter(primaryDimension);
    const dataKeys = sortedMeasures.map(({ accessor }) => accessor);
    const colorSecondY = chartConfig.secondYAxis
        ? dataKeys.findIndex((key) => { var _a; return key === ((_a = chartConfig.secondYAxis) === null || _a === void 0 ? void 0 : _a.dataKey); })
        : 0;
    const onDataPointClickInternal = useCallback((payload, eventOrIndex, event) => {
        var _a, _b, _c;
        if (typeof onDataPointClick === 'function') {
            if (payload.name) {
                const payloadValueLength = (_a = payload === null || payload === void 0 ? void 0 : payload.value) === null || _a === void 0 ? void 0 : _a.length;
                onDataPointClick(enrichEventWithDetails(event !== null && event !== void 0 ? event : eventOrIndex, {
                    value: payloadValueLength ? payload.value[1] - payload.value[0] : payload.value,
                    dataIndex: (_b = payload.index) !== null && _b !== void 0 ? _b : eventOrIndex,
                    dataKey: payloadValueLength
                        ? Object.keys(payload).filter((key) => payload.value.length
                            ? payload[key] === payload.value[1] - payload.value[0]
                            : payload[key] === payload.value && key !== 'value')[0]
                        : (_c = payload.dataKey) !== null && _c !== void 0 ? _c : Object.keys(payload).find((key) => payload[key] === payload.value && key !== 'value'),
                    payload: payload.payload
                }));
            }
            else {
                onDataPointClick(enrichEventWithDetails({}, {
                    value: eventOrIndex.value,
                    dataKey: eventOrIndex.dataKey,
                    dataIndex: eventOrIndex.index,
                    payload: eventOrIndex.payload
                }));
            }
        }
    }, [onDataPointClick]);
    const onItemLegendClick = useLegendItemClick(onLegendClick);
    const onClickInternal = useOnClickInternal(onClick);
    const isBigDataSet = (_b = (dataset === null || dataset === void 0 ? void 0 : dataset.length) > 30) !== null && _b !== void 0 ? _b : false;
    const primaryDimensionAccessor = primaryDimension === null || primaryDimension === void 0 ? void 0 : primaryDimension.accessor;
    const [yAxisWidth, legendPosition] = useLongestYAxisLabel(dataset, layout === 'vertical' ? dimensions : sortedMeasures);
    const marginChart = useChartMargin(chartConfig.margin, chartConfig.zoomingTool);
    const xAxisHeights = useObserveXAxisHeights(chartRef, layout === 'vertical' ? 1 : props.dimensions.length);
    const measureAxisProps = {
        axisLine: chartConfig.yAxisVisible,
        tickLine: tickLineConfig,
        tickFormatter: primaryMeasure === null || primaryMeasure === void 0 ? void 0 : primaryMeasure.formatter,
        interval: 0
    };
    const isRTL = useIsRTL(chartRef);
    const Placeholder = useCallback(() => {
        return React.createElement(BulletChartPlaceholder, { layout: layout, measures: measures });
    }, [layout, measures]);
    const { chartConfig: _0, dimensions: _1, measures: _2 } = rest, propsWithoutOmitted = __rest(rest, ["chartConfig", "dimensions", "measures"]);
    return (React.createElement(ChartContainer, Object.assign({ ref: componentRef, loading: loading, dataset: dataset, Placeholder: ChartPlaceholder !== null && ChartPlaceholder !== void 0 ? ChartPlaceholder : Placeholder, style: style, className: className, slot: slot, resizeDebounce: chartConfig.resizeDebounce }, propsWithoutOmitted),
        React.createElement(ComposedChartLib, { syncId: syncId, onClick: onClickInternal, stackOffset: "sign", margin: marginChart, data: dataset, layout: layout, className: typeof onDataPointClick === 'function' || typeof onClick === 'function' ? 'has-click-handler' : undefined },
            React.createElement(CartesianGrid, { vertical: chartConfig.gridVertical, horizontal: chartConfig.gridHorizontal, stroke: chartConfig.gridStroke }),
            chartConfig.xAxisVisible &&
                dimensions.map((dimension, index) => {
                    var _a;
                    let AxisComponent;
                    const axisProps = {
                        dataKey: dimension.accessor,
                        interval: (_a = dimension === null || dimension === void 0 ? void 0 : dimension.interval) !== null && _a !== void 0 ? _a : (isBigDataSet ? 'preserveStart' : 0),
                        tickLine: index < 1,
                        axisLine: index < 1,
                        allowDuplicatedCategory: index === 0
                    };
                    if (layout === 'vertical') {
                        axisProps.type = 'category';
                        axisProps.tick = React.createElement(YAxisTicks, { config: dimension });
                        axisProps.yAxisId = index;
                        axisProps.width = yAxisWidth;
                        AxisComponent = YAxis;
                        axisProps.orientation = isRTL ? 'right' : 'left';
                    }
                    else {
                        axisProps.dataKey = dimension.accessor;
                        axisProps.tick = React.createElement(XAxisTicks, { config: dimension });
                        axisProps.xAxisId = index;
                        axisProps.height = xAxisHeights[index];
                        AxisComponent = XAxis;
                        axisProps.reversed = isRTL;
                    }
                    return React.createElement(AxisComponent, Object.assign({ key: dimension.accessor }, axisProps));
                }),
            layout === 'horizontal' && (React.createElement(YAxis, Object.assign({}, measureAxisProps, { yAxisId: "primary", width: yAxisWidth, orientation: isRTL ? 'right' : 'left', tick: React.createElement(YAxisTicks, { config: primaryMeasure }) }, chartConfig.yAxisConfig))),
            layout === 'vertical' && (React.createElement(XAxis, Object.assign({}, measureAxisProps, { reversed: isRTL, xAxisId: "primary", type: "number", tick: React.createElement(XAxisTicks, { config: primaryMeasure }) }, chartConfig.xAxisConfig))),
            ((_c = chartConfig.secondYAxis) === null || _c === void 0 ? void 0 : _c.dataKey) && layout === 'horizontal' && (React.createElement(YAxis, Object.assign({ dataKey: chartConfig.secondYAxis.dataKey, axisLine: {
                    stroke: (_d = chartConfig.secondYAxis.color) !== null && _d !== void 0 ? _d : `var(--sapChart_OrderedColor_${(colorSecondY % 11) + 1})`
                }, tick: React.createElement(YAxisTicks, { config: secondaryMeasure, secondYAxisConfig: {
                        color: (_e = chartConfig.secondYAxis.color) !== null && _e !== void 0 ? _e : `var(--sapChart_OrderedColor_${(colorSecondY % 11) + 1})`
                    } }), tickLine: {
                    stroke: (_f = chartConfig.secondYAxis.color) !== null && _f !== void 0 ? _f : `var(--sapChart_OrderedColor_${(colorSecondY % 11) + 1})`
                }, 
                // eslint-disable-next-line @typescript-eslint/ban-ts-comment
                // @ts-ignore
                label: {
                    value: chartConfig.secondYAxis.name,
                    offset: 2,
                    angle: +90,
                    position: 'center'
                }, orientation: isRTL ? 'left' : 'right', interval: 0, yAxisId: "secondary" }, chartConfig.secondYAxisConfig))),
            ((_g = chartConfig.secondYAxis) === null || _g === void 0 ? void 0 : _g.dataKey) && layout === 'vertical' && (React.createElement(XAxis, Object.assign({ dataKey: chartConfig.secondYAxis.dataKey, axisLine: {
                    stroke: (_h = chartConfig.secondYAxis.color) !== null && _h !== void 0 ? _h : `var(--sapChart_OrderedColor_${(colorSecondY % 11) + 1})`
                }, tick: React.createElement(XAxisTicks, { config: secondaryMeasure, secondYAxisConfig: {
                        color: (_j = chartConfig.secondYAxis.color) !== null && _j !== void 0 ? _j : `var(--sapChart_OrderedColor_${(colorSecondY % 11) + 1})`
                    } }), tickLine: {
                    stroke: (_k = chartConfig.secondYAxis.color) !== null && _k !== void 0 ? _k : `var(--sapChart_OrderedColor_${(colorSecondY % 11) + 1})`
                }, 
                // eslint-disable-next-line @typescript-eslint/ban-ts-comment
                // @ts-ignore
                label: { value: chartConfig.secondYAxis.name, offset: 2, angle: +90, position: 'center' }, orientation: "top", interval: 0, xAxisId: "secondary", type: "number" }, chartConfig.secondXAxisConfig))),
            layout === 'horizontal' && React.createElement(XAxis, { xAxisId: 'comparisonXAxis', hide: true }),
            layout === 'vertical' && React.createElement(YAxis, { yAxisId: 'comparisonYAxis', type: 'category', hide: true }),
            chartConfig.referenceLine && (React.createElement(ReferenceLine, { stroke: chartConfig.referenceLine.color, y: layout === 'horizontal' ? chartConfig.referenceLine.value : undefined, x: layout === 'vertical' ? chartConfig.referenceLine.value : undefined, yAxisId: layout === 'horizontal' ? 'primary' : undefined, xAxisId: layout === 'vertical' ? 'primary' : undefined },
                React.createElement(Label, null, chartConfig.referenceLine.label))),
            (tooltipConfig === null || tooltipConfig === void 0 ? void 0 : tooltipConfig.active) !== false && (React.createElement(Tooltip, Object.assign({ cursor: tooltipFillOpacity, formatter: tooltipValueFormatter, contentStyle: tooltipContentStyle, labelFormatter: labelFormatter }, tooltipConfig))),
            !noLegend && (
            // eslint-disable-next-line @typescript-eslint/ban-ts-comment
            // @ts-ignore
            React.createElement(Legend, { verticalAlign: chartConfig.legendPosition, align: chartConfig.legendHorizontalAlign, onClick: onItemLegendClick, wrapperStyle: legendPosition })), sortedMeasures === null || sortedMeasures === void 0 ? void 0 :
            sortedMeasures.map((element, index) => {
                var _a, _b, _c, _d, _e, _f;
                const chartElementProps = {
                    isAnimationActive: noAnimation === false
                };
                let labelPosition = 'top';
                switch (element.type) {
                    case 'primary':
                    case 'additional':
                        chartElementProps.fillOpacity = element.opacity;
                        chartElementProps.strokeOpacity = element.opacity;
                        chartElementProps.barSize = element.width;
                        chartElementProps.onClick = onDataPointClickInternal;
                        chartElementProps.stackId = 'A';
                        chartElementProps.labelPosition = element.stackId ? 'insideTop' : 'top';
                        if (layout === 'vertical') {
                            labelPosition = 'insideRight';
                        }
                        else {
                            labelPosition = 'insideTop';
                        }
                        break;
                    case 'comparison':
                        chartElementProps.onClick = onDataPointClickInternal;
                        chartElementProps.fill = (_a = element.color) !== null && _a !== void 0 ? _a : 'black';
                        chartElementProps.strokeWidth = element.width;
                        chartElementProps.shape = React.createElement(ComparisonLine, { layout: layout });
                        chartElementProps.strokeOpacity = element.opacity;
                        chartElementProps.label = false;
                        chartElementProps.xAxisId = 'comparisonXAxis';
                        chartElementProps.yAxisId = 'comparisonYAxis';
                        chartElementProps.dot = !isBigDataSet;
                        break;
                }
                if (layout === 'vertical') {
                    chartElementProps.xAxisId = ((_b = chartConfig.secondYAxis) === null || _b === void 0 ? void 0 : _b.dataKey) === element.accessor ? 'secondary' : 'primary';
                }
                else {
                    chartElementProps.yAxisId = ((_c = chartConfig.secondYAxis) === null || _c === void 0 ? void 0 : _c.dataKey) === element.accessor ? 'secondary' : 'primary';
                }
                return (React.createElement(Bar, Object.assign({ key: element.accessor, name: (_d = element.label) !== null && _d !== void 0 ? _d : element.accessor, label: isBigDataSet ? null : React.createElement(ChartDataLabel, { config: element, chartType: 'bar', position: labelPosition }), stroke: (_e = element.color) !== null && _e !== void 0 ? _e : `var(--sapChart_OrderedColor_${(index % 11) + 1})`, fill: (_f = element.color) !== null && _f !== void 0 ? _f : `var(--sapChart_OrderedColor_${(index % 11) + 1})`, type: "monotone", dataKey: element.accessor }, chartElementProps), element.type !== 'comparison' &&
                    dataset.map((data, i) => {
                        return (React.createElement(Cell, { key: i, fill: getCellColors(element, data, index), stroke: getCellColors(element, data, index) }));
                    })));
            }),
            chartConfig.zoomingTool && (React.createElement(Brush, { y: 10, dataKey: primaryDimensionAccessor, tickFormatter: primaryDimension.formatter, stroke: ThemingParameters.sapObjectHeader_BorderColor, travellerWidth: 10, height: 20 })),
            children)));
});
BulletChart.defaultProps = {
    noLegend: false,
    noAnimation: false,
    layout: 'horizontal'
};
BulletChart.displayName = 'BulletChart';
export { BulletChart };
