var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import { ThemingParameters, useIsomorphicId } from '@ui5/webcomponents-react-base';
import React, { forwardRef } from 'react';
import { useLongestYAxisLabel } from '../../hooks/useLongestYAxisLabel';
import { usePrepareDimensionsAndMeasures } from '../../hooks/usePrepareDimensionsAndMeasures';
import { usePrepareTrendMeasures } from '../../hooks/usePrepareTrendMeasures';
import { defaultFormatter } from '../../internal/defaults';
import { ComposedChart } from '../ComposedChart';
import { ColumnChartWithTrendPlaceholder } from './Placeholder';
const dimensionDefaults = {
    formatter: defaultFormatter
};
const measureDefaults = {
    formatter: defaultFormatter,
    opacity: 1
};
const lineTooltipConfig = { wrapperStyle: { visibility: 'hidden' } };
/**
 * A `ColumnChartWithTrend` is a data visualization where each category is represented by a rectangle, with the height of the rectangle being proportional to the values being plotted amd a trend line which is displayed above the column chart.
 */
const ColumnChartWithTrend = forwardRef((props, ref) => {
    const { loading, dataset, style, className, slot, onClick, noLegend, noAnimation, onDataPointClick, onLegendClick, ChartPlaceholder } = props, rest = __rest(props, ["loading", "dataset", "style", "className", "slot", "onClick", "noLegend", "noAnimation", "onDataPointClick", "onLegendClick", "ChartPlaceholder"]);
    const syncId = useIsomorphicId();
    const chartConfig = Object.assign({ yAxisVisible: false, xAxisVisible: true, gridStroke: ThemingParameters.sapList_BorderColor, gridHorizontal: true, gridVertical: false, legendPosition: 'bottom', legendHorizontalAlign: 'left', barGap: 3, zoomingTool: false, resizeDebounce: 250 }, props.chartConfig);
    const { dimensions, measures } = usePrepareDimensionsAndMeasures(props.dimensions, props.measures, dimensionDefaults, measureDefaults);
    const { lineMeasures, columnMeasures, columnDataset } = usePrepareTrendMeasures(measures, dataset);
    const [yAxisWidth] = useLongestYAxisLabel(columnDataset, columnMeasures);
    const columnTooltipConfig = {
        formatter: (value, name, tooltipProps) => {
            const line = lineMeasures.find((currLine) => currLine.type === 'line' && currLine.accessor === tooltipProps.dataKey);
            if (line) {
                // eslint-disable-next-line @typescript-eslint/restrict-template-expressions
                return line.formatter(tooltipProps.payload[`__${line.accessor}`]);
            }
            const column = columnMeasures.find((currLine) => currLine.accessor === tooltipProps.dataKey);
            return column.formatter(value, name, tooltipProps);
        }
    };
    const { chartConfig: _0, dimensions: _1, measures: _2, tooltipConfig: _3 } = rest, propsWithoutOmitted = __rest(rest, ["chartConfig", "dimensions", "measures", "tooltipConfig"]);
    return (React.createElement("div", Object.assign({ ref: ref, style: Object.assign({ display: 'flex', flexDirection: 'column', height: style === null || style === void 0 ? void 0 : style.height, width: style === null || style === void 0 ? void 0 : style.width }, style), className: className, slot: slot }, propsWithoutOmitted),
        (dataset === null || dataset === void 0 ? void 0 : dataset.length) !== 0 && (React.createElement(ComposedChart, { className: typeof onDataPointClick === 'function' || typeof onClick === 'function' ? 'has-click-handler' : undefined, tooltipConfig: lineTooltipConfig, noAnimation: noAnimation, loading: loading, onClick: onClick, syncId: syncId, style: Object.assign(Object.assign({}, style), { height: `calc(${style === null || style === void 0 ? void 0 : style.height} * 0.2)` }), dataset: dataset, measures: lineMeasures, dimensions: dimensions, noLegend: true, chartConfig: {
                xAxisVisible: false,
                yAxisVisible: false,
                yAxisTicksVisible: false,
                gridHorizontal: false,
                yAxisLabelsVisible: false,
                yAxisWidth
            } })),
        React.createElement(ComposedChart, { onLegendClick: onLegendClick, tooltipConfig: columnTooltipConfig, noAnimation: noAnimation, noLegend: noLegend, loading: loading, onClick: onClick, syncId: syncId, ChartPlaceholder: ChartPlaceholder !== null && ChartPlaceholder !== void 0 ? ChartPlaceholder : ColumnChartWithTrendPlaceholder, dataset: columnDataset, measures: columnMeasures, dimensions: dimensions, chartConfig: chartConfig, style: Object.assign(Object.assign({}, style), { height: `calc(${style === null || style === void 0 ? void 0 : style.height} * ${(dataset === null || dataset === void 0 ? void 0 : dataset.length) !== 0 ? 0.8 : 1})` }), className: typeof onDataPointClick === 'function' || typeof onClick === 'function' ? 'has-click-handler' : undefined })));
});
ColumnChartWithTrend.defaultProps = {
    noLegend: false,
    noAnimation: false
};
ColumnChartWithTrend.displayName = 'ColumnChartWithTrend';
export { ColumnChartWithTrend };
