var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import { enrichEventWithDetails, ThemingParameters, useIsRTL, useSyncRef } from '@ui5/webcomponents-react-base';
import React, { forwardRef, useCallback } from 'react';
import { Area, Bar, Brush, CartesianGrid, Cell, ComposedChart as ComposedChartLib, Label, LabelList, Legend, Line, ReferenceLine, Tooltip, XAxis, YAxis } from 'recharts';
import { getValueByDataKey } from 'recharts/lib/util/ChartUtils';
import { useChartMargin } from '../../hooks/useChartMargin';
import { useLabelFormatter } from '../../hooks/useLabelFormatter';
import { useLegendItemClick } from '../../hooks/useLegendItemClick';
import { useLongestYAxisLabel } from '../../hooks/useLongestYAxisLabel';
import { useObserveXAxisHeights } from '../../hooks/useObserveXAxisHeights';
import { useOnClickInternal } from '../../hooks/useOnClickInternal';
import { usePrepareDimensionsAndMeasures } from '../../hooks/usePrepareDimensionsAndMeasures';
import { useTooltipFormatter } from '../../hooks/useTooltipFormatter';
import { ChartContainer } from '../../internal/ChartContainer';
import { ChartDataLabel } from '../../internal/ChartDataLabel';
import { defaultFormatter } from '../../internal/defaults';
import { tickLineConfig, tooltipContentStyle, tooltipFillOpacity } from '../../internal/staticProps';
import { getCellColors, resolvePrimaryAndSecondaryMeasures } from '../../internal/Utils';
import { XAxisTicks } from '../../internal/XAxisTicks';
import { YAxisTicks } from '../../internal/YAxisTicks';
import { ComposedChartPlaceholder } from './Placeholder';
const dimensionDefaults = {
    formatter: defaultFormatter
};
const measureDefaults = {
    formatter: defaultFormatter,
    opacity: 1
};
const ChartTypes = {
    line: Line,
    bar: Bar,
    area: Area
};
/**
 * The `ComposedChart` enables you to combine different chart types in one chart, e.g. showing bars together with lines.
 */
const ComposedChart = forwardRef((props, ref) => {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l;
    const { loading, dataset, onDataPointClick, noLegend, noAnimation, tooltipConfig, onLegendClick, onClick, layout, style, className, slot, syncId, ChartPlaceholder, children } = props, rest = __rest(props, ["loading", "dataset", "onDataPointClick", "noLegend", "noAnimation", "tooltipConfig", "onLegendClick", "onClick", "layout", "style", "className", "slot", "syncId", "ChartPlaceholder", "children"]);
    const [componentRef, chartRef] = useSyncRef(ref);
    const chartConfig = Object.assign({ yAxisLabelsVisible: true, yAxisVisible: false, xAxisVisible: true, gridStroke: ThemingParameters.sapList_BorderColor, gridHorizontal: true, gridVertical: false, legendPosition: 'bottom', legendHorizontalAlign: 'left', zoomingTool: false, resizeDebounce: 250, yAxisWidth: null, yAxisConfig: {}, xAxisConfig: {}, secondYAxisConfig: {}, secondXAxisConfig: {} }, props.chartConfig);
    const { dimensions, measures } = usePrepareDimensionsAndMeasures(props.dimensions, props.measures, dimensionDefaults, measureDefaults);
    const tooltipValueFormatter = useTooltipFormatter(measures);
    const primaryDimension = dimensions[0];
    const { primaryMeasure, secondaryMeasure } = resolvePrimaryAndSecondaryMeasures(measures, (_a = chartConfig === null || chartConfig === void 0 ? void 0 : chartConfig.secondYAxis) === null || _a === void 0 ? void 0 : _a.dataKey);
    const labelFormatter = useLabelFormatter(primaryDimension);
    const dataKeys = measures.map(({ accessor }) => accessor);
    const colorSecondY = chartConfig.secondYAxis
        ? dataKeys.findIndex((key) => { var _a; return key === ((_a = chartConfig.secondYAxis) === null || _a === void 0 ? void 0 : _a.dataKey); })
        : 0;
    const valueAccessor = (attribute) => ({ payload }) => {
        return getValueByDataKey(payload, attribute);
    };
    const onDataPointClickInternal = useCallback((payload, eventOrIndex, event) => {
        var _a, _b, _c;
        if (typeof onDataPointClick === 'function') {
            if (payload.name) {
                const payloadValueLength = (_a = payload === null || payload === void 0 ? void 0 : payload.value) === null || _a === void 0 ? void 0 : _a.length;
                onDataPointClick(enrichEventWithDetails(event !== null && event !== void 0 ? event : eventOrIndex, {
                    value: payloadValueLength ? payload.value[1] - payload.value[0] : payload.value,
                    dataIndex: (_b = payload.index) !== null && _b !== void 0 ? _b : eventOrIndex,
                    dataKey: payloadValueLength
                        ? Object.keys(payload).filter((key) => payload.value.length
                            ? payload[key] === payload.value[1] - payload.value[0]
                            : payload[key] === payload.value && key !== 'value')[0]
                        : (_c = payload.dataKey) !== null && _c !== void 0 ? _c : Object.keys(payload).find((key) => payload[key] === payload.value && key !== 'value'),
                    payload: payload.payload
                }));
            }
            else {
                onDataPointClick(enrichEventWithDetails({}, {
                    value: eventOrIndex.value,
                    dataKey: eventOrIndex.dataKey,
                    dataIndex: eventOrIndex.index,
                    payload: eventOrIndex.payload
                }));
            }
        }
    }, [onDataPointClick]);
    const onItemLegendClick = useLegendItemClick(onLegendClick);
    const onClickInternal = useOnClickInternal(onClick);
    const isBigDataSet = (_b = (dataset === null || dataset === void 0 ? void 0 : dataset.length) > 30) !== null && _b !== void 0 ? _b : false;
    const primaryDimensionAccessor = primaryDimension === null || primaryDimension === void 0 ? void 0 : primaryDimension.accessor;
    const [yAxisWidth, legendPosition] = useLongestYAxisLabel(dataset, layout === 'vertical' ? dimensions : measures);
    const marginChart = useChartMargin(chartConfig.margin, chartConfig.zoomingTool);
    const xAxisHeights = useObserveXAxisHeights(chartRef, layout === 'vertical' ? 1 : props.dimensions.length);
    const measureAxisProps = {
        axisLine: chartConfig.yAxisVisible,
        tickLine: tickLineConfig,
        tickFormatter: primaryMeasure === null || primaryMeasure === void 0 ? void 0 : primaryMeasure.formatter,
        interval: 0
    };
    const Placeholder = useCallback(() => {
        return React.createElement(ComposedChartPlaceholder, { layout: layout, measures: measures });
    }, [layout, measures]);
    const { chartConfig: _0, dimensions: _1, measures: _2 } = rest, propsWithoutOmitted = __rest(rest, ["chartConfig", "dimensions", "measures"]);
    const isRTL = useIsRTL(chartRef);
    return (React.createElement(ChartContainer, Object.assign({ ref: componentRef, loading: loading, dataset: dataset, Placeholder: ChartPlaceholder !== null && ChartPlaceholder !== void 0 ? ChartPlaceholder : Placeholder, style: style, className: className, slot: slot, resizeDebounce: chartConfig.resizeDebounce }, propsWithoutOmitted),
        React.createElement(ComposedChartLib, { syncId: syncId, onClick: onClickInternal, stackOffset: "sign", margin: marginChart, data: dataset, layout: layout, className: typeof onDataPointClick === 'function' || typeof onClick === 'function' ? 'has-click-handler' : undefined },
            React.createElement(CartesianGrid, { vertical: chartConfig.gridVertical, horizontal: chartConfig.gridHorizontal, stroke: chartConfig.gridStroke }),
            dimensions.map((dimension, index) => {
                var _a, _b;
                let AxisComponent;
                const axisProps = {
                    dataKey: dimension.accessor,
                    interval: (_a = dimension === null || dimension === void 0 ? void 0 : dimension.interval) !== null && _a !== void 0 ? _a : (isBigDataSet ? 'preserveStart' : 0),
                    tickLine: index < 1,
                    axisLine: index < 1,
                    allowDuplicatedCategory: index === 0
                };
                if (layout === 'vertical') {
                    axisProps.type = 'category';
                    axisProps.visible = false;
                    axisProps.hide = !chartConfig.yAxisVisible;
                    axisProps.tick = React.createElement(YAxisTicks, { config: dimension });
                    axisProps.yAxisId = index;
                    axisProps.width = (_b = chartConfig.yAxisWidth) !== null && _b !== void 0 ? _b : yAxisWidth;
                    AxisComponent = YAxis;
                    axisProps.orientation = isRTL ? 'right' : 'left';
                }
                else {
                    axisProps.dataKey = dimension.accessor;
                    axisProps.tick = React.createElement(XAxisTicks, { config: dimension });
                    axisProps.hide = !chartConfig.xAxisVisible;
                    axisProps.xAxisId = index;
                    axisProps.height = xAxisHeights[index];
                    AxisComponent = XAxis;
                    axisProps.reversed = isRTL;
                }
                return React.createElement(AxisComponent, Object.assign({ key: dimension.accessor }, axisProps));
            }),
            layout === 'horizontal' && (React.createElement(YAxis, Object.assign({}, measureAxisProps, { yAxisId: "primary", width: (_c = chartConfig.yAxisWidth) !== null && _c !== void 0 ? _c : yAxisWidth, orientation: isRTL ? 'right' : 'left', tick: chartConfig.yAxisLabelsVisible ? React.createElement(YAxisTicks, { config: primaryMeasure }) : false }, chartConfig.yAxisConfig))),
            layout === 'vertical' && (React.createElement(XAxis, Object.assign({}, measureAxisProps, { reversed: isRTL, xAxisId: "primary", type: "number", tick: React.createElement(XAxisTicks, { config: primaryMeasure }) }, chartConfig.xAxisConfig))),
            ((_d = chartConfig.secondYAxis) === null || _d === void 0 ? void 0 : _d.dataKey) && layout === 'horizontal' && (React.createElement(YAxis, Object.assign({ dataKey: chartConfig.secondYAxis.dataKey, axisLine: {
                    stroke: (_e = chartConfig.secondYAxis.color) !== null && _e !== void 0 ? _e : `var(--sapChart_OrderedColor_${(colorSecondY % 11) + 1})`
                }, tick: React.createElement(YAxisTicks, { config: secondaryMeasure, secondYAxisConfig: {
                        color: (_f = chartConfig.secondYAxis.color) !== null && _f !== void 0 ? _f : `var(--sapChart_OrderedColor_${(colorSecondY % 11) + 1})`
                    } }), tickLine: {
                    stroke: (_g = chartConfig.secondYAxis.color) !== null && _g !== void 0 ? _g : `var(--sapChart_OrderedColor_${(colorSecondY % 11) + 1})`
                }, 
                // eslint-disable-next-line @typescript-eslint/ban-ts-comment
                // @ts-ignore
                label: {
                    value: chartConfig.secondYAxis.name,
                    offset: 2,
                    angle: +90,
                    position: 'center'
                }, orientation: isRTL ? 'left' : 'right', interval: 0, yAxisId: "secondary" }, chartConfig.secondYAxisConfig))),
            ((_h = chartConfig.secondYAxis) === null || _h === void 0 ? void 0 : _h.dataKey) && layout === 'vertical' && (React.createElement(XAxis, Object.assign({ dataKey: chartConfig.secondYAxis.dataKey, axisLine: {
                    stroke: (_j = chartConfig.secondYAxis.color) !== null && _j !== void 0 ? _j : `var(--sapChart_OrderedColor_${(colorSecondY % 11) + 1})`
                }, tick: React.createElement(XAxisTicks, { config: secondaryMeasure, secondYAxisConfig: {
                        color: (_k = chartConfig.secondYAxis.color) !== null && _k !== void 0 ? _k : `var(--sapChart_OrderedColor_${(colorSecondY % 11) + 1})`
                    } }), tickLine: {
                    stroke: (_l = chartConfig.secondYAxis.color) !== null && _l !== void 0 ? _l : `var(--sapChart_OrderedColor_${(colorSecondY % 11) + 1})`
                }, 
                // eslint-disable-next-line @typescript-eslint/ban-ts-comment
                // @ts-ignore
                label: { value: chartConfig.secondYAxis.name, offset: 2, angle: +90, position: 'center' }, orientation: "top", interval: 0, xAxisId: "secondary", type: "number" }, chartConfig.secondXAxisConfig))),
            chartConfig.referenceLine && (React.createElement(ReferenceLine, { stroke: chartConfig.referenceLine.color, y: layout === 'horizontal' ? chartConfig.referenceLine.value : undefined, x: layout === 'vertical' ? chartConfig.referenceLine.value : undefined, yAxisId: layout === 'horizontal' ? 'primary' : undefined, xAxisId: layout === 'vertical' ? 'primary' : undefined },
                React.createElement(Label, null, chartConfig.referenceLine.label))),
            (tooltipConfig === null || tooltipConfig === void 0 ? void 0 : tooltipConfig.active) !== false && (React.createElement(Tooltip, Object.assign({ cursor: tooltipFillOpacity, formatter: tooltipValueFormatter, contentStyle: tooltipContentStyle, labelFormatter: labelFormatter }, tooltipConfig))),
            !noLegend && (
            // eslint-disable-next-line @typescript-eslint/ban-ts-comment
            // @ts-ignore
            React.createElement(Legend, { verticalAlign: chartConfig.legendPosition, align: chartConfig.legendHorizontalAlign, onClick: onItemLegendClick, wrapperStyle: legendPosition })), measures === null || measures === void 0 ? void 0 :
            measures.map((element, index) => {
                var _a, _b, _c, _d, _e, _f, _g;
                const ChartElement = ChartTypes[element.type];
                const chartElementProps = {
                    isAnimationActive: noAnimation === false
                };
                let labelPosition = 'top';
                switch (element.type) {
                    case 'line':
                        chartElementProps.activeDot = {
                            onClick: onDataPointClickInternal
                        };
                        chartElementProps.strokeWidth = element.width;
                        chartElementProps.strokeOpacity = element.opacity;
                        chartElementProps.dot = (_a = element.showDot) !== null && _a !== void 0 ? _a : !isBigDataSet;
                        break;
                    case 'bar':
                        chartElementProps.fillOpacity = element.opacity;
                        chartElementProps.strokeOpacity = element.opacity;
                        chartElementProps.barSize = element.width;
                        chartElementProps.onClick = onDataPointClickInternal;
                        chartElementProps.stackId = (_b = element.stackId) !== null && _b !== void 0 ? _b : undefined;
                        chartElementProps.labelPosition = element.stackId ? 'insideTop' : 'top';
                        if (layout === 'vertical') {
                            labelPosition = 'insideRight';
                        }
                        else {
                            labelPosition = 'insideTop';
                        }
                        break;
                    case 'area':
                        chartElementProps.dot = !isBigDataSet;
                        chartElementProps.fillOpacity = 0.3;
                        chartElementProps.strokeOpacity = element.opacity;
                        chartElementProps.onClick = onDataPointClickInternal;
                        chartElementProps.strokeWidth = element.width;
                        break;
                }
                if (layout === 'vertical') {
                    chartElementProps.xAxisId = ((_c = chartConfig.secondYAxis) === null || _c === void 0 ? void 0 : _c.dataKey) === element.accessor ? 'secondary' : 'primary';
                }
                else {
                    chartElementProps.yAxisId = ((_d = chartConfig.secondYAxis) === null || _d === void 0 ? void 0 : _d.dataKey) === element.accessor ? 'secondary' : 'primary';
                }
                return (React.createElement(ChartElement, Object.assign({ key: element.accessor, name: (_e = element.label) !== null && _e !== void 0 ? _e : element.accessor, label: element.type === 'bar' || isBigDataSet ? undefined : (React.createElement(ChartDataLabel, { config: element, chartType: element.type, position: labelPosition })), stroke: (_f = element.color) !== null && _f !== void 0 ? _f : `var(--sapChart_OrderedColor_${(index % 11) + 1})`, fill: (_g = element.color) !== null && _g !== void 0 ? _g : `var(--sapChart_OrderedColor_${(index % 11) + 1})`, type: "monotone", dataKey: element.accessor }, chartElementProps), element.type === 'bar' && (React.createElement(React.Fragment, null,
                    React.createElement(LabelList, { data: dataset, valueAccessor: valueAccessor(element.accessor), content: React.createElement(ChartDataLabel, { config: element, chartType: "column", position: 'insideTop' }) }),
                    dataset.map((data, i) => {
                        return (React.createElement(Cell, { key: i, fill: getCellColors(element, data, index), stroke: getCellColors(element, data, index) }));
                    })))));
            }),
            chartConfig.zoomingTool && (React.createElement(Brush, { y: 10, dataKey: primaryDimensionAccessor, tickFormatter: primaryDimension.formatter, stroke: ThemingParameters.sapObjectHeader_BorderColor, travellerWidth: 10, height: 20 })),
            children)));
});
ComposedChart.defaultProps = {
    noLegend: false,
    noAnimation: false,
    layout: 'horizontal'
};
ComposedChart.displayName = 'ComposedChart';
export { ComposedChart };
