var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import { enrichEventWithDetails, ThemingParameters, useIsRTL, useSyncRef } from '@ui5/webcomponents-react-base';
import React, { forwardRef, useCallback, useRef } from 'react';
import { Brush, CartesianGrid, Label, Legend, Line, LineChart as LineChartLib, ReferenceLine, Tooltip, XAxis, YAxis } from 'recharts';
import { useChartMargin } from '../../hooks/useChartMargin';
import { useLabelFormatter } from '../../hooks/useLabelFormatter';
import { useLegendItemClick } from '../../hooks/useLegendItemClick';
import { useLongestYAxisLabel } from '../../hooks/useLongestYAxisLabel';
import { useObserveXAxisHeights } from '../../hooks/useObserveXAxisHeights';
import { usePrepareDimensionsAndMeasures } from '../../hooks/usePrepareDimensionsAndMeasures';
import { useTooltipFormatter } from '../../hooks/useTooltipFormatter';
import { ChartContainer } from '../../internal/ChartContainer';
import { ChartDataLabel } from '../../internal/ChartDataLabel';
import { defaultFormatter } from '../../internal/defaults';
import { tickLineConfig, tooltipContentStyle, tooltipFillOpacity, xAxisPadding } from '../../internal/staticProps';
import { resolvePrimaryAndSecondaryMeasures } from '../../internal/Utils';
import { XAxisTicks } from '../../internal/XAxisTicks';
import { YAxisTicks } from '../../internal/YAxisTicks';
import { LineChartPlaceholder } from './Placeholder';
const dimensionDefaults = {
    formatter: defaultFormatter
};
const measureDefaults = {
    formatter: defaultFormatter,
    width: 1,
    opacity: 1
};
/**
 * A `LineChart` is a type of chart used to show information that changes over time - it connects multiple dots.
 */
const LineChart = forwardRef((props, ref) => {
    var _a, _b, _c, _d, _e, _f;
    const { dataset, loading, noLegend, noAnimation, tooltipConfig, onDataPointClick, onLegendClick, onClick, style, className, slot, syncId, ChartPlaceholder, children } = props, rest = __rest(props, ["dataset", "loading", "noLegend", "noAnimation", "tooltipConfig", "onDataPointClick", "onLegendClick", "onClick", "style", "className", "slot", "syncId", "ChartPlaceholder", "children"]);
    const chartConfig = Object.assign({ yAxisVisible: false, xAxisVisible: true, gridStroke: ThemingParameters.sapList_BorderColor, gridHorizontal: true, gridVertical: false, legendPosition: 'bottom', legendHorizontalAlign: 'left', zoomingTool: false, resizeDebounce: 250, yAxisTicksVisible: true, yAxisConfig: {}, xAxisConfig: {}, secondYAxisConfig: {} }, props.chartConfig);
    const { dimensions, measures } = usePrepareDimensionsAndMeasures(props.dimensions, props.measures, dimensionDefaults, measureDefaults);
    const tooltipValueFormatter = useTooltipFormatter(measures);
    const primaryDimension = dimensions[0];
    const { primaryMeasure, secondaryMeasure } = resolvePrimaryAndSecondaryMeasures(measures, (_a = chartConfig === null || chartConfig === void 0 ? void 0 : chartConfig.secondYAxis) === null || _a === void 0 ? void 0 : _a.dataKey);
    const labelFormatter = useLabelFormatter(primaryDimension);
    const [componentRef, chartRef] = useSyncRef(ref);
    const dataKeys = measures.map(({ accessor }) => accessor);
    const colorSecondY = chartConfig.secondYAxis
        ? dataKeys.findIndex((key) => { var _a; return key === ((_a = chartConfig.secondYAxis) === null || _a === void 0 ? void 0 : _a.dataKey); })
        : 0;
    const onItemLegendClick = useLegendItemClick(onLegendClick);
    const preventOnClickCall = useRef(0);
    const onDataPointClickInternal = useCallback((payload, eventOrIndex) => {
        var _a, _b;
        if (eventOrIndex.dataKey && typeof onDataPointClick === 'function') {
            preventOnClickCall.current = 2;
            onDataPointClick(enrichEventWithDetails({}, {
                value: eventOrIndex.value,
                dataKey: eventOrIndex.dataKey,
                dataIndex: eventOrIndex.index,
                payload: eventOrIndex.payload
            }));
        }
        else if (typeof onClick === 'function' && preventOnClickCall.current === 0) {
            onClick(enrichEventWithDetails(eventOrIndex, {
                payload: (_b = (_a = payload === null || payload === void 0 ? void 0 : payload.activePayload) === null || _a === void 0 ? void 0 : _a[0]) === null || _b === void 0 ? void 0 : _b.payload,
                activePayloads: payload === null || payload === void 0 ? void 0 : payload.activePayload
            }));
        }
        if (preventOnClickCall.current > 0) {
            preventOnClickCall.current -= 1;
        }
    }, [onDataPointClick, preventOnClickCall.current]);
    const isBigDataSet = (_b = (dataset === null || dataset === void 0 ? void 0 : dataset.length) > 30) !== null && _b !== void 0 ? _b : false;
    const primaryDimensionAccessor = primaryDimension === null || primaryDimension === void 0 ? void 0 : primaryDimension.accessor;
    const [yAxisWidth, legendPosition] = useLongestYAxisLabel(dataset, measures);
    const marginChart = useChartMargin(chartConfig.margin, chartConfig.zoomingTool);
    const xAxisHeights = useObserveXAxisHeights(chartRef, props.dimensions.length);
    const { chartConfig: _0, dimensions: _1, measures: _2 } = rest, propsWithoutOmitted = __rest(rest, ["chartConfig", "dimensions", "measures"]);
    const isRTL = useIsRTL(chartRef);
    return (React.createElement(ChartContainer, Object.assign({ dataset: dataset, loading: loading, Placeholder: ChartPlaceholder !== null && ChartPlaceholder !== void 0 ? ChartPlaceholder : LineChartPlaceholder, ref: componentRef, style: style, className: className, slot: slot, resizeDebounce: chartConfig.resizeDebounce }, propsWithoutOmitted),
        React.createElement(LineChartLib, { syncId: syncId, margin: marginChart, data: dataset, onClick: onDataPointClickInternal, className: typeof onDataPointClick === 'function' ? 'has-click-handler' : undefined },
            React.createElement(CartesianGrid, { vertical: chartConfig.gridVertical, horizontal: chartConfig.gridHorizontal, stroke: chartConfig.gridStroke }),
            dimensions.map((dimension, index) => {
                var _a;
                return (React.createElement(XAxis, Object.assign({ key: dimension.accessor, dataKey: dimension.accessor, xAxisId: index, interval: (_a = dimension === null || dimension === void 0 ? void 0 : dimension.interval) !== null && _a !== void 0 ? _a : (isBigDataSet ? 'preserveStart' : 0), tick: React.createElement(XAxisTicks, { config: dimension }), tickLine: index < 1, axisLine: index < 1, height: chartConfig.xAxisVisible ? xAxisHeights[index] : 0, padding: xAxisPadding, allowDuplicatedCategory: index === 0, reversed: isRTL }, chartConfig.xAxisConfig)));
            }),
            React.createElement(YAxis, Object.assign({ orientation: isRTL === true ? 'right' : 'left', axisLine: chartConfig.yAxisVisible, tickLine: tickLineConfig, yAxisId: "left", tickFormatter: primaryMeasure === null || primaryMeasure === void 0 ? void 0 : primaryMeasure.formatter, interval: 0, tick: chartConfig.yAxisTicksVisible && React.createElement(YAxisTicks, { config: primaryMeasure }), width: yAxisWidth }, chartConfig.yAxisConfig)),
            ((_c = chartConfig.secondYAxis) === null || _c === void 0 ? void 0 : _c.dataKey) && (React.createElement(YAxis, Object.assign({ dataKey: chartConfig.secondYAxis.dataKey, axisLine: {
                    stroke: (_d = chartConfig.secondYAxis.color) !== null && _d !== void 0 ? _d : `var(--sapChart_OrderedColor_${(colorSecondY % 11) + 1})`
                }, tick: React.createElement(YAxisTicks, { config: secondaryMeasure, secondYAxisConfig: {
                        color: (_e = chartConfig.secondYAxis.color) !== null && _e !== void 0 ? _e : `var(--sapChart_OrderedColor_${(colorSecondY % 11) + 1})`
                    } }), tickLine: {
                    stroke: (_f = chartConfig.secondYAxis.color) !== null && _f !== void 0 ? _f : `var(--sapChart_OrderedColor_${(colorSecondY % 11) + 1})`
                }, 
                // eslint-disable-next-line @typescript-eslint/ban-ts-comment
                // @ts-ignore
                label: { value: chartConfig.secondYAxis.name, offset: 2, angle: +90, position: 'center' }, orientation: isRTL === true ? 'left' : 'right', yAxisId: "right", interval: 0 }, chartConfig.secondYAxisConfig))),
            measures.map((element, index) => {
                var _a, _b, _c, _d;
                return (React.createElement(Line, { dot: (_a = element.showDot) !== null && _a !== void 0 ? _a : !isBigDataSet, yAxisId: ((_b = chartConfig.secondYAxis) === null || _b === void 0 ? void 0 : _b.dataKey) === element.accessor ? 'right' : 'left', key: element.accessor, name: (_c = element.label) !== null && _c !== void 0 ? _c : element.accessor, strokeOpacity: element.opacity, 
                    // eslint-disable-next-line @typescript-eslint/ban-ts-comment
                    // @ts-ignore
                    label: isBigDataSet ? false : React.createElement(ChartDataLabel, { config: element, chartType: "line", position: "top" }), type: "monotone", dataKey: element.accessor, stroke: (_d = element.color) !== null && _d !== void 0 ? _d : `var(--sapChart_OrderedColor_${(index % 11) + 1})`, strokeWidth: element.width, activeDot: { onClick: onDataPointClickInternal }, isAnimationActive: noAnimation === false }));
            }),
            !noLegend && (
            // eslint-disable-next-line @typescript-eslint/ban-ts-comment
            // @ts-ignore
            React.createElement(Legend, { verticalAlign: chartConfig.legendPosition, align: chartConfig.legendHorizontalAlign, onClick: onItemLegendClick, wrapperStyle: legendPosition })),
            chartConfig.referenceLine && (React.createElement(ReferenceLine, { stroke: chartConfig.referenceLine.color, y: chartConfig.referenceLine.value, yAxisId: 'left' },
                React.createElement(Label, null, chartConfig.referenceLine.label))),
            (tooltipConfig === null || tooltipConfig === void 0 ? void 0 : tooltipConfig.active) !== false && (React.createElement(Tooltip, Object.assign({ cursor: tooltipFillOpacity, formatter: tooltipValueFormatter, contentStyle: tooltipContentStyle, labelFormatter: labelFormatter }, tooltipConfig))),
            chartConfig.zoomingTool && (React.createElement(Brush, { y: 10, dataKey: primaryDimensionAccessor, tickFormatter: primaryDimension.formatter, stroke: ThemingParameters.sapObjectHeader_BorderColor, travellerWidth: 10, height: 20 })),
            children)));
});
LineChart.defaultProps = {
    noLegend: false,
    noAnimation: false
};
LineChart.displayName = 'LineChart';
export { LineChart };
