var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import { enrichEventWithDetails, ThemingParameters } from '@ui5/webcomponents-react-base';
import clsx from 'clsx';
import React, { forwardRef, useCallback, useMemo } from 'react';
import { createUseStyles } from 'react-jss';
import { getValueByDataKey } from 'recharts/lib/util/ChartUtils';
import { ChartContainer } from '../../internal/ChartContainer';
import { defaultFormatter } from '../../internal/defaults';
import { BarChartPlaceholder } from '../BarChart/Placeholder';
const resolveColor = (index, color = null) => {
    var _a;
    if (color) {
        return (_a = ThemingParameters[color]) !== null && _a !== void 0 ? _a : color;
    }
    return ThemingParameters[`sapChart_Sequence_${(index % 11) + 1}`];
};
const MicroBarChartStyles = {
    container: {
        display: 'flex',
        flexDirection: 'column',
        overflow: 'hidden',
        fontFamily: ThemingParameters.sapFontFamily,
        fontWeight: 'normal',
        width: '100%',
        height: '100%',
        justifyContent: 'space-around'
    },
    barContainer: {
        cursor: 'auto'
    },
    barContainerActive: {
        '&:active': { opacity: '0.3 !important' },
        cursor: 'pointer'
    },
    labelContainer: {
        display: 'flex',
        justifyContent: 'space-between'
    },
    valueContainer: {
        display: 'flex',
        backgroundColor: ThemingParameters.sapContent_Placeholderloading_Background
    },
    valueBar: { height: '100%' },
    label: {
        color: ThemingParameters.sapContent_LabelColor,
        display: 'block',
        overflow: 'hidden',
        whiteSpace: 'nowrap',
        textOverflow: 'ellipsis',
        fontSize: ThemingParameters.sapFontSmallSize,
        maxWidth: '70%'
    },
    text: {
        paddingLeft: '6px',
        display: 'inline-block',
        overflow: 'hidden',
        fontSize: ThemingParameters.sapFontSmallSize,
        boxSizing: 'border-box',
        whiteSpace: 'nowrap',
        textOverflow: 'ellipsis',
        color: ThemingParameters.sapTextColor,
        textAlign: 'right'
    }
};
const useStyles = createUseStyles(MicroBarChartStyles, { name: 'MicroBarChart' });
/**
 * The `MicroBarChart` compares different values of the same category to each other by displaying them in a compact way.
 */
const MicroBarChart = forwardRef((props, ref) => {
    const { loading, dataset, onDataPointClick, style, className, slot, ChartPlaceholder } = props, rest = __rest(props, ["loading", "dataset", "onDataPointClick", "style", "className", "slot", "ChartPlaceholder"]);
    const classes = useStyles();
    const dimension = useMemo(() => (Object.assign({ formatter: defaultFormatter }, props.dimension)), [props.dimension]);
    const measure = useMemo(() => (Object.assign({ formatter: defaultFormatter }, props.measure)), [props.measure]);
    const maxValue = useMemo(() => {
        var _a;
        if (dataset) {
            const maxDatasetValue = Math.max(...dataset === null || dataset === void 0 ? void 0 : dataset.map((item) => getValueByDataKey(item, measure.accessor)));
            return (_a = props.maxValue) !== null && _a !== void 0 ? _a : maxDatasetValue;
        }
        return 0;
    }, [dataset, measure, props.maxValue]);
    const barHeight = (measure === null || measure === void 0 ? void 0 : measure.width) ? `${measure.width}px` : '4px';
    const onBarClick = useCallback((item, index) => (e) => {
        if (typeof onDataPointClick === 'function') {
            onDataPointClick(enrichEventWithDetails(e, {
                dataKey: measure.accessor,
                value: getValueByDataKey(item, measure.accessor),
                payload: item,
                dataIndex: index
            }));
        }
    }, [measure.accessor, onDataPointClick]);
    const barContainerClasses = clsx(classes.barContainer, onDataPointClick && classes.barContainerActive);
    const { maxValue: _0, dimension: _1, measure: _2 } = rest, propsWithoutOmitted = __rest(rest, ["maxValue", "dimension", "measure"]);
    return (React.createElement(ChartContainer, Object.assign({ dataset: dataset, loading: loading, Placeholder: ChartPlaceholder !== null && ChartPlaceholder !== void 0 ? ChartPlaceholder : BarChartPlaceholder, ref: ref, style: style, className: className, slot: slot, resizeDebounce: 250 }, propsWithoutOmitted),
        React.createElement("div", { className: classes.container }, dataset === null || dataset === void 0 ? void 0 : dataset.map((item, index) => {
            var _a, _b;
            const dimensionValue = getValueByDataKey(item, dimension.accessor);
            const measureValue = getValueByDataKey(item, measure.accessor);
            const formattedDimension = dimension.formatter(dimensionValue);
            let formattedMeasure = '';
            if (!measure.hideDataLabel) {
                if (measure.DataLabel) {
                    formattedMeasure = React.createElement(measure.DataLabel, {
                        value: measureValue,
                        config: measure,
                        formattedValue: measure.formatter(measureValue)
                    });
                }
                else {
                    formattedMeasure = measure.formatter(measureValue);
                }
            }
            return (React.createElement("div", { key: dimensionValue, className: barContainerClasses, onClick: onBarClick(item, index) },
                React.createElement("div", { className: classes.labelContainer },
                    React.createElement("span", { className: classes.label, title: formattedDimension }, formattedDimension),
                    React.createElement("span", { className: classes.text, title: formattedMeasure }, measure.hideDataLabel ? '' : formattedMeasure)),
                React.createElement("div", { className: classes.valueContainer, style: {
                        opacity: (_a = measure === null || measure === void 0 ? void 0 : measure.opacity) !== null && _a !== void 0 ? _a : 1,
                        height: barHeight
                    } },
                    React.createElement("div", { className: classes.valueBar, style: {
                            width: `${(measureValue / maxValue) * 100}%`,
                            backgroundColor: resolveColor(index, (_b = measure === null || measure === void 0 ? void 0 : measure.colors) === null || _b === void 0 ? void 0 : _b[index])
                        } }))));
        }))));
});
MicroBarChart.displayName = 'MicroBarChart';
export { MicroBarChart };
