var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import { enrichEventWithDetails, ThemingParameters } from '@ui5/webcomponents-react-base';
import React, { forwardRef, useCallback, useRef } from 'react';
import { Legend, PolarAngleAxis, PolarGrid, PolarRadiusAxis, Radar, RadarChart as RadarChartLib, Tooltip } from 'recharts';
import { useLabelFormatter } from '../../hooks/useLabelFormatter';
import { useLegendItemClick } from '../../hooks/useLegendItemClick';
import { usePrepareDimensionsAndMeasures } from '../../hooks/usePrepareDimensionsAndMeasures';
import { useTooltipFormatter } from '../../hooks/useTooltipFormatter';
import { ChartContainer } from '../../internal/ChartContainer';
import { ChartDataLabel } from '../../internal/ChartDataLabel';
import { defaultFormatter } from '../../internal/defaults';
import { tooltipContentStyle, tooltipFillOpacity } from '../../internal/staticProps';
import { PieChartPlaceholder } from '../PieChart/Placeholder';
const dimensionDefaults = {
    formatter: (d) => d
};
const measureDefaults = {
    formatter: defaultFormatter,
    opacity: 0.5
};
/**
 * A radar or spider or web chart is a two-dimensional chart type designed to plot one or more series of values over multiple quantitative variables.
 */
const RadarChart = forwardRef((props, ref) => {
    const { loading, dataset, noLegend, noAnimation, tooltipConfig, onDataPointClick, onLegendClick, onClick, style, className, slot, ChartPlaceholder, children } = props, rest = __rest(props, ["loading", "dataset", "noLegend", "noAnimation", "tooltipConfig", "onDataPointClick", "onLegendClick", "onClick", "style", "className", "slot", "ChartPlaceholder", "children"]);
    const chartConfig = Object.assign({ legendPosition: 'bottom', legendHorizontalAlign: 'center', dataLabel: true, polarGridType: 'circle', resizeDebounce: 250 }, props.chartConfig);
    const { dimensions, measures } = usePrepareDimensionsAndMeasures(props.dimensions, props.measures, dimensionDefaults, measureDefaults);
    const tooltipValueFormatter = useTooltipFormatter(measures);
    const primaryDimension = dimensions[0];
    const labelFormatter = useLabelFormatter(primaryDimension);
    const primaryDimensionAccessor = primaryDimension === null || primaryDimension === void 0 ? void 0 : primaryDimension.accessor;
    const onItemLegendClick = useLegendItemClick(onLegendClick);
    const preventOnClickCall = useRef(false);
    const onClickInternal = useCallback((payload, event) => {
        var _a, _b;
        if (typeof onClick === 'function' && !preventOnClickCall.current) {
            onClick(enrichEventWithDetails(event, {
                payload: (_b = (_a = payload === null || payload === void 0 ? void 0 : payload.activePayload) === null || _a === void 0 ? void 0 : _a[0]) === null || _b === void 0 ? void 0 : _b.payload,
                activePayloads: payload === null || payload === void 0 ? void 0 : payload.activePayload
            }));
        }
        if (preventOnClickCall.current) {
            preventOnClickCall.current = false;
        }
    }, [onClick, preventOnClickCall.current]);
    const onDataPointClickInternal = useCallback((payload, eventOrIndex) => {
        if (eventOrIndex.value && onDataPointClick) {
            onDataPointClick(enrichEventWithDetails({}, {
                value: eventOrIndex.value,
                dataKey: eventOrIndex.dataKey,
                name: eventOrIndex.payload.label,
                dataIndex: eventOrIndex.index,
                payload: eventOrIndex.payload
            }));
            preventOnClickCall.current = true;
        }
    }, [onDataPointClick, preventOnClickCall.current]);
    const { chartConfig: _0, dimensions: _1, measures: _2 } = rest, propsWithoutOmitted = __rest(rest, ["chartConfig", "dimensions", "measures"]);
    return (React.createElement(ChartContainer, Object.assign({ dataset: dataset, ref: ref, loading: loading, Placeholder: ChartPlaceholder !== null && ChartPlaceholder !== void 0 ? ChartPlaceholder : PieChartPlaceholder, style: style, className: className, slot: slot, resizeDebounce: chartConfig.resizeDebounce }, propsWithoutOmitted),
        React.createElement(RadarChartLib, { onClick: onClickInternal, data: dataset, margin: chartConfig.margin, className: typeof onDataPointClick === 'function' ? 'has-click-handler' : undefined },
            React.createElement(PolarGrid, { gridType: chartConfig.polarGridType }),
            React.createElement(PolarAngleAxis, { dataKey: primaryDimensionAccessor, tickFormatter: primaryDimension === null || primaryDimension === void 0 ? void 0 : primaryDimension.formatter, tick: {
                    fill: ThemingParameters.sapContent_LabelColor
                } }),
            React.createElement(PolarRadiusAxis, null),
            measures.map((element, index) => {
                var _a, _b, _c;
                return (React.createElement(Radar, { key: element.accessor, activeDot: { onClick: onDataPointClickInternal }, name: (_a = element.label) !== null && _a !== void 0 ? _a : element.accessor, dataKey: element.accessor, stroke: (_b = element.color) !== null && _b !== void 0 ? _b : `var(--sapChart_OrderedColor_${(index % 11) + 1})`, fill: (_c = element.color) !== null && _c !== void 0 ? _c : `var(--sapChart_OrderedColor_${(index % 11) + 1})`, fillOpacity: element.opacity, label: React.createElement(ChartDataLabel, { config: element, chartType: "radar", position: 'outside' }), isAnimationActive: noAnimation === false }));
            }),
            (tooltipConfig === null || tooltipConfig === void 0 ? void 0 : tooltipConfig.active) !== false && (React.createElement(Tooltip, Object.assign({ cursor: tooltipFillOpacity, formatter: tooltipValueFormatter, contentStyle: tooltipContentStyle, labelFormatter: labelFormatter }, tooltipConfig))),
            !noLegend && (
            // eslint-disable-next-line @typescript-eslint/ban-ts-comment
            // @ts-ignore
            React.createElement(Legend, { verticalAlign: chartConfig.legendPosition, align: chartConfig.legendHorizontalAlign, onClick: onItemLegendClick })),
            children)));
});
RadarChart.defaultProps = {
    noLegend: false,
    noAnimation: false
};
RadarChart.displayName = 'RadarChart';
export { RadarChart };
