var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import { enrichEventWithDetails, ThemingParameters, useIsRTL, useSyncRef } from '@ui5/webcomponents-react-base';
import React, { forwardRef, useCallback, useRef } from 'react';
import { CartesianGrid, Label, Legend, ReferenceLine, Scatter, ScatterChart as ScatterChartLib, Tooltip, XAxis, YAxis, ZAxis } from 'recharts';
import { useChartMargin } from '../../hooks/useChartMargin';
import { useLegendItemClick } from '../../hooks/useLegendItemClick';
import { useLongestYAxisLabel } from '../../hooks/useLongestYAxisLabel';
import { useObserveXAxisHeights } from '../../hooks/useObserveXAxisHeights';
import { usePrepareDimensionsAndMeasures } from '../../hooks/usePrepareDimensionsAndMeasures';
import { useTooltipFormatter } from '../../hooks/useTooltipFormatter';
import { ChartContainer } from '../../internal/ChartContainer';
import { defaultFormatter } from '../../internal/defaults';
import { tickLineConfig, tooltipContentStyle, tooltipFillOpacity, xAxisPadding } from '../../internal/staticProps';
import { XAxisTicks } from '../../internal/XAxisTicks';
import { YAxisTicks } from '../../internal/YAxisTicks';
import { ScatterChartPlaceholder } from './Placeholder';
const measureDefaults = {
    formatter: defaultFormatter
};
/**
 *
 * A `ScatterChart` is a data visualization that displays multiple circles (bubbles) in a two-dimensional plot.
 *
 * Most commonly, a scatter chart displays the values of three numeric variables,where each observation's data is
 * shown by a circle, while the horizontal and vertical positions of the bubble show the values of two other variables.
 */
const ScatterChart = forwardRef((props, ref) => {
    var _a, _b;
    const { dataset, loading, noLegend, noAnimation, tooltipConfig, onDataPointClick, onLegendClick, onClick, style, className, slot, ChartPlaceholder, children } = props, rest = __rest(props, ["dataset", "loading", "noLegend", "noAnimation", "tooltipConfig", "onDataPointClick", "onLegendClick", "onClick", "style", "className", "slot", "ChartPlaceholder", "children"]);
    const chartConfig = Object.assign({ yAxisVisible: false, xAxisVisible: true, gridStroke: ThemingParameters.sapList_BorderColor, gridHorizontal: true, gridVertical: false, legendPosition: 'bottom', legendHorizontalAlign: 'left', zoomingTool: false, resizeDebounce: 250 }, props.chartConfig);
    const { measures } = usePrepareDimensionsAndMeasures([], props.measures, {}, measureDefaults);
    const tooltipValueFormatter = useTooltipFormatter(measures);
    const [componentRef, chartRef] = useSyncRef(ref);
    const preventOnClickCall = useRef(false);
    const onItemLegendClick = useLegendItemClick(onLegendClick);
    const onClickInternal = useCallback((payload, event) => {
        var _a, _b;
        if (typeof onClick === 'function' && !preventOnClickCall.current) {
            onClick(enrichEventWithDetails(event, {
                payload: (_b = (_a = payload === null || payload === void 0 ? void 0 : payload.activePayload) === null || _a === void 0 ? void 0 : _a[0]) === null || _b === void 0 ? void 0 : _b.payload,
                activePayloads: payload === null || payload === void 0 ? void 0 : payload.activePayload
            }));
        }
        if (preventOnClickCall.current) {
            preventOnClickCall.current = false;
        }
    }, [onClick, preventOnClickCall.current]);
    const onDataPointClickInternal = useCallback((payload, eventOrIndex, event) => {
        if (payload && onDataPointClick) {
            onDataPointClick(enrichEventWithDetails(event, {
                value: payload.node,
                dataKey: payload.zAxis.dataKey,
                dataIndex: eventOrIndex,
                payload: payload.payload
            }));
            preventOnClickCall.current = true;
        }
    }, [onDataPointClick, preventOnClickCall.current]);
    const isBigDataSet = (_a = (dataset === null || dataset === void 0 ? void 0 : dataset.length) > 30) !== null && _a !== void 0 ? _a : false;
    const xMeasure = measures.find(({ axis }) => axis === 'x');
    const yMeasure = measures.find(({ axis }) => axis === 'y');
    const zMeasure = measures.find(({ axis }) => axis === 'z');
    const [yAxisWidth, legendPosition] = useLongestYAxisLabel(dataset === null || dataset === void 0 ? void 0 : dataset[0].data, [yMeasure]);
    const xAxisHeights = useObserveXAxisHeights(chartRef, 1);
    const marginChart = useChartMargin(chartConfig.margin, chartConfig.zoomingTool);
    const { chartConfig: _0, measures: _1 } = rest, propsWithoutOmitted = __rest(rest, ["chartConfig", "measures"]);
    const isRTL = useIsRTL(chartRef);
    return (React.createElement(ChartContainer, Object.assign({ dataset: dataset, loading: loading, Placeholder: ChartPlaceholder !== null && ChartPlaceholder !== void 0 ? ChartPlaceholder : ScatterChartPlaceholder, ref: componentRef, style: style, className: className, slot: slot, resizeDebounce: chartConfig.resizeDebounce }, propsWithoutOmitted),
        React.createElement(ScatterChartLib, { onClick: onClickInternal, margin: marginChart, className: typeof onDataPointClick === 'function' ? 'has-click-handler' : undefined },
            React.createElement(CartesianGrid, { vertical: chartConfig.gridVertical, horizontal: chartConfig.gridHorizontal, stroke: chartConfig.gridStroke }),
            chartConfig.xAxisVisible && (React.createElement(XAxis, { type: 'number', key: xMeasure === null || xMeasure === void 0 ? void 0 : xMeasure.accessor, name: xMeasure === null || xMeasure === void 0 ? void 0 : xMeasure.label, dataKey: xMeasure === null || xMeasure === void 0 ? void 0 : xMeasure.accessor, xAxisId: 0, interval: (_b = xMeasure === null || xMeasure === void 0 ? void 0 : xMeasure.interval) !== null && _b !== void 0 ? _b : (isBigDataSet ? 'preserveStart' : 0), tick: React.createElement(XAxisTicks, { config: xMeasure }), padding: xAxisPadding, height: xAxisHeights[0], reversed: isRTL, 
                // eslint-disable-next-line @typescript-eslint/ban-ts-comment
                // @ts-ignore
                label: (xMeasure === null || xMeasure === void 0 ? void 0 : xMeasure.label) ? { value: xMeasure === null || xMeasure === void 0 ? void 0 : xMeasure.label, dy: 15, position: 'insideRight' } : 0 })),
            React.createElement(YAxis
            // eslint-disable-next-line @typescript-eslint/ban-ts-comment
            // @ts-ignore
            , { 
                // eslint-disable-next-line @typescript-eslint/ban-ts-comment
                // @ts-ignore
                label: (yMeasure === null || yMeasure === void 0 ? void 0 : yMeasure.label)
                    ? { value: yMeasure === null || yMeasure === void 0 ? void 0 : yMeasure.label, angle: -90, position: isRTL ? 'insideRight' : 'insideLeft' }
                    : false, type: 'number', name: yMeasure === null || yMeasure === void 0 ? void 0 : yMeasure.label, axisLine: chartConfig.yAxisVisible, tickLine: tickLineConfig, key: yMeasure === null || yMeasure === void 0 ? void 0 : yMeasure.accessor, dataKey: yMeasure === null || yMeasure === void 0 ? void 0 : yMeasure.accessor, tickFormatter: yMeasure === null || yMeasure === void 0 ? void 0 : yMeasure.formatter, interval: 0, tick: React.createElement(YAxisTicks, { config: yMeasure }), width: (yMeasure === null || yMeasure === void 0 ? void 0 : yMeasure.label) ? yAxisWidth + 10 : yAxisWidth, margin: (yMeasure === null || yMeasure === void 0 ? void 0 : yMeasure.label) ? { left: 200 } : 0, orientation: isRTL === true ? 'right' : 'left' }),
            React.createElement(ZAxis, { name: zMeasure === null || zMeasure === void 0 ? void 0 : zMeasure.label, dataKey: zMeasure === null || zMeasure === void 0 ? void 0 : zMeasure.accessor, range: [0, 5000], key: zMeasure === null || zMeasure === void 0 ? void 0 : zMeasure.accessor }), dataset === null || dataset === void 0 ? void 0 :
            dataset.map((dataSet, index) => {
                var _a;
                return (React.createElement(Scatter, { className: typeof onDataPointClick === 'function' ? 'has-click-handler' : undefined, 
                    // eslint-disable-next-line @typescript-eslint/ban-ts-comment
                    // @ts-ignore
                    onMouseDown: onDataPointClickInternal, opacity: dataSet.opacity, data: dataSet === null || dataSet === void 0 ? void 0 : dataSet.data, name: dataSet === null || dataSet === void 0 ? void 0 : dataSet.label, key: dataSet === null || dataSet === void 0 ? void 0 : dataSet.label, fill: (_a = dataSet === null || dataSet === void 0 ? void 0 : dataSet.color) !== null && _a !== void 0 ? _a : `var(--sapChart_OrderedColor_${(index % 11) + 1})`, isAnimationActive: noAnimation === false }));
            }),
            !noLegend && (
            // eslint-disable-next-line @typescript-eslint/ban-ts-comment
            // @ts-ignore
            React.createElement(Legend, { verticalAlign: chartConfig.legendPosition, align: chartConfig.legendHorizontalAlign, onClick: onItemLegendClick, wrapperStyle: legendPosition })),
            chartConfig.referenceLine && (React.createElement(ReferenceLine, { stroke: chartConfig.referenceLine.color, y: chartConfig.referenceLine.value },
                React.createElement(Label, null, chartConfig.referenceLine.label))),
            chartConfig.referenceLineX && (React.createElement(ReferenceLine, { stroke: chartConfig.referenceLineX.color, x: chartConfig.referenceLineX.value },
                React.createElement(Label, null, chartConfig.referenceLineX.label))),
            (tooltipConfig === null || tooltipConfig === void 0 ? void 0 : tooltipConfig.active) !== false && (React.createElement(Tooltip, Object.assign({ cursor: tooltipFillOpacity, formatter: tooltipValueFormatter, contentStyle: tooltipContentStyle }, tooltipConfig))),
            children)));
});
ScatterChart.defaultProps = {
    noLegend: false,
    noAnimation: false
};
ScatterChart.displayName = 'ScatterChart';
export { ScatterChart };
